package com.bytesw.bytebot.service;

import com.bytesw.bytebot.etl.beans.GoalBean;
import com.bytesw.bytebot.etl.model.Goal;
import com.bytesw.bytebot.repository.*;
import com.bytesw.xdf.exception.AlreadyExistsException;
import com.bytesw.xdf.exception.NotFoundException;
import com.bytesw.xdf.service.XDFService;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;
import java.util.Optional;

@Service
@Transactional
@Log4j2
public class GoalService extends XDFService<Goal, GoalBean, Long> {

    @Autowired
    private GoalRepository goalRepository;

    @Autowired
    private GoalForActionsRepository goalForActionsRepository;

    @Autowired
    private AgentRepository agentRepository;

    @Autowired
    private GoalForActionService goalForActionService;

    protected GoalService(GoalRepository repository) {
        super(repository);
    }

    @Override
    protected Goal toModel(Goal model, GoalBean bean) {
        if (model == null) {
            model = new Goal();
        }
        BeanUtils.copyProperties(bean, model);
        return model;
    }

    @Override
    protected GoalBean toBean(Goal model) {
        GoalBean bean = new GoalBean();
        BeanUtils.copyProperties(model, bean);
        return bean;
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public GoalBean save(Map<String, Object> body) {

        Goal goal = new Goal();

        if (body.isEmpty()) {
            throw new NotFoundException("No se encuentran parametros.");
        }

        Long agenId = Long.valueOf(body.get("AgentId").toString());
        String ident = body.get("Ident").toString();

        if (!agentRepository.existsById(agenId)) {
            throw new NotFoundException("Agente no registrado.");
        }

        Optional<Goal> goalOptional= goalRepository.findByAgentAndIdent(agenId, ident);

        if (goalOptional.isPresent()) {
            throw new AlreadyExistsException("Goal ya existe en la base de datos.");
        }

        goal.setAgenId(agenId);
        goal.setIdentifier(ident);
        goal = goalRepository.save(goal);
        goalForActionService.save(goal.getId(), (List) body.get("actionList"));
        return toBean(goal);

    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED)
    public void delete(Long goalId) {

        if (!goalRepository.existsById(goalId)){
            throw new NotFoundException("Goal no registrado.");
        }

        goalForActionService.delete(goalId);
        goalRepository.deleteById(goalId);
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public void updateGoalForActions(Map<String, Object> body, Long goalId) {

        if (body.isEmpty()) {
            throw new NotFoundException("No se encuentran parametros.");
        }

        if (!goalRepository.existsById(goalId)) {
            throw new NotFoundException("Goal no se encuenta registrado");
        }

        Long agenId = Long.valueOf(body.get("AgentId").toString());
        String ident = body.get("Ident").toString();

        if (!agentRepository.existsById(agenId)) {
            throw new NotFoundException("Agente no registrado.");
        }

        Goal goal = new Goal();
        goal.setId(goalId);
        goal.setAgenId(agenId);
        goal.setIdentifier(ident);

        goal = goalRepository.save(goal);
        goalForActionService.update(goal.getId(), (List) body.get("actionList"));
    }
}